; ukazkovy program ucebnice programovani PIC
; verze 1.6
; lze pouze prelozit, obsahuje direktivy prekladace, ale nic nedela
; obsahuje definice hardware jednotky
; inicializace procesoru a zakladni osetreni preruseni
; znovunaplneni a obsluha preruseni, obsluha displeje
; generace nahodnych cisel, D/A prevodnik, a vystupni kmitocet pro
; piezosirenku
; obsluha klavesnice a rizeni procesu
; meziprocesni vazby a prevody ciselnych soustav

 LIST P=16F84, R=DEC                    ; direktivy prekladace

 include<p16f84.inc>                    ; definice nazvu registru

                                        ; definice zapojeni hardware
KL_R1     EQU     3                     ; portb
KL_R2     EQU     2                     ; portb
KL_R3     EQU     1                     ; portb
KL_S1     EQU     4                     ; portb
KL_S2     EQU     5                     ; portb
KL_S3     EQU     6                     ; portb
KL_S4     EQU     7                     ; portb
IN_RND    EQU     0                     ; portb
AD_OUT    EQU     4                     ; porta
OUT_STB   EQU     3                     ; porta
OUT_D     EQU     2                     ; portb
OUT_CLK   EQU     3                     ; portb
OUTPIP    EQU     2                     ; porta
AN1       EQU     1                     ; porta
AN2       EQU     0                     ; porta

KL_CLR    EQU     10                    ; definice neciselnych klaves
KL_ENT    EQU     11                    ;
NULL      EQU     12                    ; zadny znak nebyl stisten

;--------------------------------------------------------------------
; promenne
;--------------------------------------------------------------------

OffRam    EQU     0x0C                  ; offset pameti pro PIC16F84
Mem_W     EQU     OffRam                ; pamet pracovniho registru
Mem_Stat  EQU     OffRam+1              ; pamet registru STATUS
Disp1     EQU     OffRam+2              ; promenne znaku displeje
Disp2     EQU     OffRam+3              ; ok
PorDisp   EQU     OffRam+4              ; poradi zobrazovaneho znaku
Pom       EQU     OffRam+5              ; pomocna promenna
Cykl      EQU     OffRam+6              ; promenna pocitani cyklu
RND1      EQU     OffRam+7              ; generace nahodnych cisel
RND2      EQU     OffRam+8              ;
RND3      EQU     OffRam+9              ; ok
ProcOn    EQU     OffRam+10             ; promenna rizeni procesu
Run       EQU     0                     ; povoleni obsluhy procesu
RND       EQU     1                     ; proces nahodneho cisla
Pip       EQU     2                     ; proces pipnuti
TlPull    EQU     3                     ; priznak pridrzene klavesy
OutLevel  EQU     OffRam+11             ; vystupni uroven
PWM       EQU     OffRam+12             ; ridici promenna PWM
DirProc   EQU     OffRam+13             ; promenna identifikace procesu
TmrDelay  EQU     OffRam+14             ; promenna pocitadla zpozdeni
TmrPip    EQU     OffRam+15             ; promenna delky pipnuti

;--------------------------------------------------------------------
; kod programu
;--------------------------------------------------------------------

                ORG     0       
                GOTO    Start           ; skok na zacatek programu

                ORG     4               ;
                GOTO    IntProc         ; skok na obsluhu preruseni

Segment                                 ; procedura prevede znak na 
                                        ; sedmisegmentovy kod
                ADDWF   PCL,F           ; pricteno poradi znaku
                RETLW   0x40            ; 0
                RETLW   0x79            ; 1
                RETLW   0x24            ; 2
                RETLW   0x30            ; 3
                RETLW   0x19            ; 4
                RETLW   0x12            ; 5
                RETLW   0x02            ; 6
                RETLW   0x78            ; 7
                RETLW   0x00            ; 8
                RETLW   0x10            ; 9

Anoda                                   ; procedura vrati v registru W
                                        ; pozici anody rozsviceneho znaku
                ADDWF   PCL,F           ;
                RETLW   0x02            ; 1. znak
                RETLW   0x01            ; 2. znak

MakeProc                                ; spravce procesu
                MOVF    DirProc,W       ; vyber aktualniho procesu
                ADDWF   PCL,F           ; ok
                GOTO    PR_Init         ; inicializace
                GOTO    PR_Wait         ; proces cekani 1s
                GOTO    PR_KlvDisp      ; proces klaves a displeje
                GOTO    PR_SetLevel     ; nastaveni vystupni urovne

RdKlaves                                ; cteni klavesnice
                CLRF    Pom             ; clr poradi znaku
                MOVLW   0xF7            ; predefinice testu 1.radku
                MOVWF   PORTB           ;   
                MOVLW   3               ; definice poctu radku
                MOVWF   Cykl            ; ok
KlvCykl         BTFSS   PORTB,4         ; test 1 sloupce
                GOTO    DetKlv          ;
                INCF    Pom,F           ;
                BTFSS   PORTB,5         ; test 2 sloupce
                GOTO    DetKlv          ;
                INCF    Pom,F           ;
                BTFSS   PORTB,6         ; test 3 sloupce
                GOTO    DetKlv          ;
                INCF    Pom,F           ;
                BTFSS   PORTB,7         ; test 4 sloupce
                GOTO    DetKlv          ;
                INCF    Pom,F           ;
                RRF     PORTB,F         ;
                DECFSZ  Cykl,F          ; cykl detekce klavesy
                GOTO    KlvCykl         ;
DetKlv          MOVF    Pom,W           ; detekce znaku
                ADDWF   PCL,F           ;
                RETLW   1               ;
                RETLW   2               ;
                RETLW   3               ;
                RETLW   4               ;
                RETLW   5               ;
                RETLW   6               ;
                RETLW   7               ;
                RETLW   8               ;
                RETLW   9               ;
                RETLW   0               ;
                RETLW   KL_CLR          ;
                RETLW   KL_ENT          ;
                RETLW   NULL            ;

;--------------------------------------------------------------------
; hlavni program
;--------------------------------------------------------------------

Start                                   ; inicializace procesoru
                BSF     STATUS,RP0      ; registrova sada 1
                MOVLW   0xC5            ; nastaveni OPTION
                MOVWF   OPTION_REG      ; ok
                CLRF    TRISA           ; piny PORTA jako vystupy
                MOVLW   0xF1            ; RB0 a RB4 az RB7 vstupy
                MOVWF   TRISB           ; jinak vystupy
                BCF     STATUS,RP0      ; registriva sada 0
                CLRF    PORTA           ; cely PORTA do 0
                CLRF    PorDisp         ; pocatecni znak
                CLRF    OutLevel        ; clr vystupni urovne
                CLRF    ProcOn          ; stop vsech procesu
                CLRF    DirProc         ; pocatek procesni inicializace
                MOVLW   3               ; pocatecni prvocisla
                MOVWF   RND1            ;
                MOVLW   7               ;
                MOVWF   RND2            ; ok
                MOVLW   0xFF            ; cely PORTB do 1
                MOVWF   PORTB           ; ok
                MOVLW   100             ; hodnota prvniho naplneni TMR0
                MOVWF   TMR0            ; ok
                MOVLW   8               ; inicializace displeje
                MOVWF   Disp1           ;
                MOVWF   Disp2           ; ok
                MOVLW   0xB0            ; povoleni preruseni
                MOVWF   INTCON          ; ok
Main
                BTFSS   ProcOn,Pip      ; proces pipnuti
                GOTO    NoPip           ;
                MOVLW   0x04            ; pozice bitu outpip
                XORWF   PORTA,F         ; inverze bitu OUTPIP
NoPip           BTFSS   ProcOn,Pip      ; pri klidu piskani vystup
                BCF     PORTA,OUTPIP    ; do 0
                MOVLW   Disp1           ; vyber znaku podle poradi
                ADDWF   PorDisp,W       ;
                MOVWF   FSR             ;
                MOVF    INDF,W          ; ok
                CALL    Segment         ; prevod znaku na sedmisegment
                CALL    SendZnak        ; prenos znaku na 4094
                MOVLW   0xFC            ; zhasnuti anod znaku
                ANDWF   PORTA,F         ;
                BCF     PORTA,OUT_STB   ; zapisovy pulz znaku
                BSF     PORTA,OUT_STB   ;
                BCF     PORTA,OUT_STB   ; ok
                MOVF    PorDisp,W       ; rozsviceni aktualni anody
                CALL    Anoda           ;
                IORWF   PORTA,F         ; ok
                INCF    PorDisp,F       ; dalsi znak
                MOVLW   2               ; test druheho znaku
                SUBWF   PorDisp,W       ;
                BTFSC   STATUS,Z        ;
                CLRF    PorDisp         ; ok
                MOVF    RND1,W          ; generator pseudonahody
                ADDWF   RND2,W          ;
                MOVWF   RND3            ;
                MOVF    RND2,W          ;
                ADDWF   RND3,W          ;
                MOVWF   RND1            ;
                MOVF    RND3,W          ;
                ADDWF   RND1,W          ;
                MOVWF   RND2            ; ok
                INCF    PWM,F           ; inkrement urovne prevodu
                MOVLW   100             ; test urovne 100
                SUBWF   PWM,W           ;
                BTFSC   STATUS,Z        ;
                CLRF    PWM             ; ok
                MOVF    PWM,W           ; test meze urovne
                SUBWF   OutLevel,W      ;
                BTFSS   STATUS,C        ; prekrocena ?
                GOTO    PWM_L           ;
                BSF     PORTA,AD_OUT    ;
                GOTO    M_Proc          ;
PWM_L           BCF     PORTA,AD_OUT    ;
M_Proc          BTFSS   ProcOn,Run      ; test procesu obsluhy
                GOTO    Main            ;
                BCF     ProcOn,Run      ; shozeni priznaku obsluhy
                BTFSS   ProcOn,Pip      ; test procesu pipnuti
                GOTO    PrNoPip         ; nepiska se
                INCF    TmrPip,F        ; zapocteni useku 10ms
                MOVLW   10              ; pipnuti 100ms
                SUBWF   TmrPip,W        ; test konce pipnuti
                BTFSC   STATUS,Z        ;
                BCF     ProcOn,Pip      ; ok
PrNoPip         INCF    TmrDelay,F      ; zapocteni useku 10ms
                GOTO    MakeProc        ; skok na spravce procesu

PR_Init                                 ; proces prvotni inicializace
                CLRF    TmrDelay        ; reset pocitadla zpozdeni
                MOVLW   1               ; proces cekani
                MOVWF   DirProc         ; ok
                GOTO    Main            ;

PR_Wait                                 ; proces cekani 1s
                MOVLW   100             ; 100 useku 10ms
                SUBWF   TmrDelay,W      ; test konce cekani
                BTFSS   STATUS,Z        ;
                GOTO    Main            ; neni konec cekani
                CALL    KlvPipnuti      ; pipnuti po inicializaci
                MOVLW   2               ; proces klaves a displeje
                MOVWF   DirProc         ;
                GOTO    Main            ;

PR_KlvDisp                              ; proces klaves a displeje
                BTFSS   ProcOn,RND      ; proces prenosu nahody
                GOTO    NoRND           ;
                BCF     ProcOn,RND      ; uvolneni dalsiho generovani
                MOVF    RND3,W          ; RND3 do Pom
                MOVWF   Pom             ; ok
C_100           MOVLW   100             ; upava cisla na 0 az 99
                SUBWF   Pom,W           ;
                BTFSS   STATUS,C        ; je vetsi nez 100
                GOTO    EndC_100        ;
                MOVWF   Pom             ;
                GOTO    C_100           ; cykl odpoctu
EndC_100        MOVF    Pom,W           ; nastaveni cisla
                MOVWF   OutLevel        ;
NoRND           MOVF    OutLevel,W      ; prenos urovne na displej
                MOVWF   Pom             ; pamet urovne do Pom
                CLRF    Cykl            ;
Rad_10          MOVLW   10              ; vypocet radu desitek
                SUBWF   Pom,W           ;
                BTFSS   STATUS,C        ;
                GOTO    End_10          ;
                MOVWF   Pom             ;
                INCF    Cykl,F          ;
                GOTO    Rad_10          ;
End_10          MOVF    Cykl,W          ;
                MOVWF   Disp2           ;
                MOVF    Pom,W           ; prenos radu jednotek
                MOVWF   Disp1           ;
                CALL    RdKlaves        ; nacteni klavesnice
                MOVWF   Pom             ; pamet stisteneho znaku
                MOVLW   NULL            ; test nestistene klavesnice
                SUBWF   Pom,W           ;
                BTFSS   STATUS,Z        ;
                GOTO    Tst_CLR         ; skok na test CLR
                BCF     ProcOn,TlPull   ; clr priznaku stistene klavesy
                GOTO    Main            ;
Tst_CLR         BTFSC   ProcOn,TlPull   ; test pridrzene klavesy
                GOTO    Main            ; klavesa jiz byla sejmuta
                MOVLW   KL_CLR          ; test klavesy CLR
                SUBWF   Pom,W           ;
                BTFSS   STATUS,Z        ;
                GOTO    Tst_ENT         ; skok na test ENT
                BSF     ProcOn,TlPull   ; priznak sejmuti klavesy
                CLRF    OutLevel        ; clear nastavene urovne
                CALL    KlvPipnuti      ; klavesove pipnuti
                GOTO    Main            ;
Tst_ENT         MOVLW   KL_ENT          ; test klavesy ENT
                SUBWF   Pom,W           ;
                BTFSC   STATUS,Z        ;
                GOTO    Main            ; zde nic klavesa nedela
                MOVLW   3               ; nastaveni procesu SetLevel
                MOVWF   DirProc         ; ok
                GOTO    Main            ;

PR_SetLevel                             ; proces nastaveni OutLevel
                CALL    RdKlaves        ; nacteni klavesnice
                MOVWF   Pom             ; pamet stisteneho znaku
                MOVLW   NULL            ; test nestistene klavesnice
                SUBWF   Pom,W           ;
                BTFSS   STATUS,Z        ;
                GOTO    Tst_ENT2        ; skok na test ENT
                BCF     ProcOn,TlPull   ; clr priznaku stistene klavesy
                GOTO    Main            ;
Tst_ENT2        BTFSC   ProcOn,TlPull   ; test pridrzene klavesy
                GOTO    Main            ; klavesa jiz byla sejmuta
                MOVLW   KL_ENT          ; test klavesy ENT
                SUBWF   Pom,W           ;
                BTFSS   STATUS,Z        ;
                GOTO    Tst_CLR2        ; test CLR
                BSF     ProcOn,TlPull   ; klavesa byla sejmuta
                CALL    KlvPipnuti      ; pipnuti klavesy
                CALL    MakeOutLevel    ; vypocte binarni hodnotu z dec
                MOVWF   OutLevel        ; zapis
                MOVLW   2               ; proces klavesnice
                MOVWF   DirProc         ;
                GOTO    Main            ;
Tst_CLR2        MOVLW   KL_CLR          ; test klavesy CLR
                SUBWF   Pom,W           ;
                BTFSC   STATUS,Z        ;
                GOTO    Main            ; klavesa nic nedela
                MOVF    Disp1,W         ; posun znaku
                MOVWF   Disp2           ;
                MOVF    Pom,W           ;
                MOVWF   Disp1           ; ok
                BSF     ProcOn,TlPull   ; klavesa sejmuta
                CALL    KlvPipnuti      ;
                GOTO    Main            ;

;--------------------------------------------------------------------
; obsluzne procedury
;--------------------------------------------------------------------

SendZnak                                ; znak z W na vystup
                MOVWF   Pom             ; znak do Pom
                MOVLW   8               ; cykl osmi bitu
                MOVWF   Cykl            ;
Loop_Disp       BCF     PORTB,OUT_D     ; prenos na 4094
                RLF     Pom,F           ;
                BTFSC   STATUS,C        ;
                BSF     PORTB,OUT_D     ;
                BCF     PORTB,OUT_CLK   ;
                BSF     PORTB,OUT_CLK   ;
                DECFSZ  Cykl,F          ; test konce cyklu
                GOTO    Loop_Disp       ; smycka prenosu
                RETURN                  ;

KlvPipnuti                              ; spusteni pipnuti klaves
                CLRF    TmrPip          ; pocatek mereni intervapu pip
                BSF     ProcOn,Pip      ; pocatek piskani
                RETURN                  ;

MakeOutLevel                            ; prevod cisla dec na bin
                CLRF    Pom             ; clr pomocne promenne
                MOVF    Disp2,W         ;
                MOVWF   Cykl            ; rad desitek do parametru cyklu
M_Des           MOVF    Cykl,F          ; test nuly desitek
                BTFSC   STATUS,Z        ;
                GOTO    EndM_Des        ; konec vypoctu desitek
                MOVLW   10              ;
                ADDWF   Pom,F           ; pripocteni desitky
                DECF    Cykl,F          ;
                GOTO    M_Des           ;
EndM_Des        MOVF    Disp1,W         ; pricteni jednotek
                ADDWF   Pom,W           ;
                RETURN                  ;

IntProc                                 ; obsluha preruseni
                MOVWF   Mem_W           ; ulozeni registru W
                MOVF    STATUS,W        ; ulozeni obsahu STATUS
                MOVWF   Mem_Stat        ; ok
                BTFSS   INTCON,T0IF     ; bylo preruseni od TMR0
                GOTO    TestINT0        ; nebylo, skok na obsluhu RB0
                MOVLW   100             ; znovunaplneni TMR0
                MOVWF   TMR0            ; ok
                BCF     INTCON,T0IF     ; clr priznaku preruseni TMR0
                BSF     ProcOn,Run      ; proces obsluhy
                GOTO    EndInt          ; skok na ukonceni preruseni
TestINT0        BCF     INTCON,INTF     ; clr priznaku preruseni INT0
                BSF     ProcOn,RND      ; proces nahodneho cisla
EndInt          MOVF    Mem_Stat,W      ; obnova stavu STATUS
                MOVWF   STATUS          ; ok
                SWAPF   Mem_W,F         ; obnova registru W
                SWAPF   Mem_W,W         ; ok
                RETFIE                  ; navrat z preruseni

                END                     ; konec programu

